import os
import rdkit
from rdkit import Chem
import pandas as pd
import shutil
import csv
from .cal_ifp import cal_ifp

def calInchiKey2smiFromTxt(smiles_txt_path):
    f = open(smiles_txt_path , 'r')
    smis = f.readlines()
    smis = [i[:-1] for i in smis]
    inchiKeys = [Chem.MolToInchiKey(Chem.MolFromSmiles(smi)) for smi in smis]
    inchiKey2smi = dict(zip(inchiKeys, smis))
    return inchiKey2smi
def remove_last_number_and_hyphen(elements):
    cleaned_elements = []

    for element in elements:
        # 找到最后一个 '-' 的位置
        last_hyphen_index = element.rfind('-')
        
        if last_hyphen_index != -1:
            # 去除最后的数字和前面的 '-'
            cleaned = element[:last_hyphen_index]
        else:
            cleaned = element  # 如果没有 '-', 返回原始元素
        
        cleaned_elements.append(cleaned)

    return cleaned_elements

def get_first_column_all_elements(file_path):
    first_column_elements = []

    with open(file_path, 'r', encoding='utf-8') as file:
        for line in file:
            # 分割行，获取第一列元素
            first_column = line.split('\t')[0]  # 假设列之间用制表符分隔
            first_column_elements.append(first_column.strip())
    first_column_elements = first_column_elements[1:]
    return list(set(remove_last_number_and_hyphen(first_column_elements)))
def find_max_value_element(data):
    max_element = None
    max_value = float('-inf')

    for item in data:
        # 分割字符串，获取关键字和数值
        key, value_str = item.split('\t')
        value = float(value_str)  # 将数值字符串转换为浮点数

        # 更新最大值和对应元素
        if value > max_value:
            max_value = value
            max_element = item

    return max_element

def find_lines_with_biggestScore(file_path, search_string):
    matching_lines = []

    with open(file_path, 'r', encoding='utf-8') as file:
        for line in file:
            if search_string in line:
                matching_lines.append(line.strip())
    max_element = find_max_value_element(matching_lines)
    return max_element

def remove_last_number_and_hyphen_single(input_string):
    # 找到最后一个 '-' 的位置
    last_hyphen_index = input_string.rfind('-')
    
    if last_hyphen_index != -1:
        # 去除最后的数字和前面的 '-'
        cleaned_string = input_string[:last_hyphen_index]
    else:
        cleaned_string = input_string  # 如果没有 '-', 返回原始字符串
    
    return cleaned_string
def find_lines_with_min_num(file_path, search_string):
    matching_lines = []

    with open(file_path, 'r', encoding='utf-8') as file:
        for line in file:
            if search_string in line:
                matching_lines.append(line.strip())
    nums = min([int(get_last_number(i.split('\t')[0])) for i in matching_lines])
    # print(matching_lines)
    return nums
def get_last_number(input_string):
    # 找到最后一个 '-' 的位置
    last_hyphen_index = input_string.rfind('-')
    
    if last_hyphen_index != -1:
        # 提取最后一个 '-' 之后的部分
        last_part = input_string[last_hyphen_index + 1:]
        # 返回最后部分的数字
        return last_part
    else:
        return None  
    
def write_dict_to_csv(data_dict, csv_filename):
    # 打开或创建 CSV 文件
    with open(csv_filename, mode='w', newline='', encoding='utf-8') as csvfile:
        writer = csv.writer(csvfile)
        
        # 写入标题
        writer.writerow(['InchiKey', 'Smiles'])
        
        # 写入字典的键值对
        for key, value in data_dict.items():
            writer.writerow([key, value])

def get_final_smi_score(carsidock_prediction_file , smiles_txt_path):
    all_elements = get_first_column_all_elements(carsidock_prediction_file)
    inchiKey2smi =calInchiKey2smiFromTxt(smiles_txt_path)
    write_dict_to_csv(inchiKey2smi , os.path.join(os.path.dirname(carsidock_prediction_file) , 'inchiKey2smi.csv'))
    final_smiles = []
    final_scores = []
    for element in all_elements:
        # print(element)
        biggestScore = find_lines_with_biggestScore(carsidock_prediction_file , element)
        final_smile = inchiKey2smi[remove_last_number_and_hyphen_single(biggestScore.split('\t')[0])]
        final_smiles.append(final_smile)
        final_score = float(remove_last_number_and_hyphen_single(biggestScore.split('\t')[1]))
        final_scores.append(round(final_score,4))
    data = pd.DataFrame()
    data['smiles'] = final_smiles
    data['scores'] = final_scores
    data = data.sort_values(by='scores',ascending=False)
    data.to_csv(os.path.join(os.path.dirname(carsidock_prediction_file) , 'carsidock_prediction.csv') , index=False)


def get_final_ligand_poses(carsidock_prediction_file , protein_file_name , input_protein):
    all_elements = get_first_column_all_elements(carsidock_prediction_file)
    if not os.path.exists(os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses')):
        os.mkdir(os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses'))
    if not os.path.exists(os.path.join(os.path.dirname(carsidock_prediction_file) , 'top10_poses_interaction')):
        os.mkdir(os.path.join(os.path.dirname(carsidock_prediction_file) , 'top10_poses_interaction'))
        
    for element in all_elements:
        biggestScore = find_lines_with_biggestScore(carsidock_prediction_file , element)
        num = int(get_last_number(biggestScore.split('\t')[0]))
        num_min = find_lines_with_min_num(carsidock_prediction_file , element)
        index_num = num-num_min
        sdf_file = os.path.join(os.path.dirname(carsidock_prediction_file) , '%s.sdf'%element)
        output_file = os.path.join(os.path.dirname(carsidock_prediction_file) ,'ligand_poses', '%s.sdf'%element)
        suppl = Chem.SDMolSupplier(sdf_file)
        os.remove(sdf_file)
        mol = suppl[index_num]
        writer = Chem.SDWriter(output_file)
        writer.write(mol)
        writer.close()
    
    data_carsidock_prediction = pd.read_csv(os.path.join(os.path.dirname(carsidock_prediction_file) , 'carsidock_prediction.csv'))
    top10_smiles = list(data_carsidock_prediction['smiles'])[:10]
    data_map = pd.read_csv(os.path.join(os.path.dirname(carsidock_prediction_file) , 'inchiKey2smi.csv'))
    smi2inchikey = dict(zip(list(data_map['Smiles']), list(data_map['InchiKey'])))
    top10_inchikey = [smi2inchikey[i] for i in top10_smiles]
    top10_poses_path = os.path.join(os.path.dirname(carsidock_prediction_file) , 'top10_poses_interaction')
    protein_path = os.path.join(os.path.dirname(input_protein) , protein_file_name)
    for i in range(len(top10_inchikey)):
        tmp_inchikey = top10_inchikey[i]
        ligand_path = os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses', '%s.sdf'%tmp_inchikey)
        top10_poses_path_tmp = os.path.join(top10_poses_path , 'top%s'%i)
        if not os.path.exists(top10_poses_path_tmp):
            os.mkdir(top10_poses_path_tmp)
        path_output = top10_poses_path_tmp
       # compose_protein_and_ligand(protein_path , ligand_path , path_output )
        cal_ifp(protein_path , ligand_path , path_output , i)
    
    compress_folder(os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses'), os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses'))
    compress_folder(os.path.join(os.path.dirname(carsidock_prediction_file) , 'top10_poses_interaction'), os.path.join(os.path.dirname(carsidock_prediction_file) , 'top10_poses_interaction'))
    os.system('rm -rf %s'%os.path.join(os.path.dirname(carsidock_prediction_file) , 'ligand_poses'))

def compress_folder(input_path, output_path):
    # 检查输入路径是否存在
    if not os.path.exists(input_path):
        raise FileNotFoundError(f"The input path '{input_path}' does not exist.")
    
    # 压缩文件夹
    try:
        # Create a zip file from the input folder
        shutil.make_archive(os.path.splitext(output_path)[0], 'zip', input_path)
        print(f"Successfully compressed '{input_path}' into '{output_path}.zip'.")
    except Exception as e:
        print(f"An error occurred during compression: {e}")


import os
import subprocess
import pandas as pd
from .util_post_process import get_final_smi_score , get_final_ligand_poses
import sys
import argparse


def read_smiles_from_file(text_file):
    molecules_dict = {}
    
    with open(text_file, 'r') as file:
        lines = file.readlines()
        for line in lines:
            molecule_info = line.split()
            molecule_name = molecule_info[1]
            molecule_smiles = molecule_info[0]
            molecules_dict[molecule_name] = molecule_smiles 
    return molecules_dict


def process_karmadock_prediction_files(text_file, csv_file,karmadock_score_top_smile_txt , topN=50):
    
    df = pd.read_csv(csv_file)
    sorted_df = df.sort_values(by='karma_score', ascending=False).iloc[:topN,:]
    smile_ids = list(sorted_df['pdb_id'])

    name2smile = read_smiles_from_file(text_file)
    
    
    top_smiles = []
    for i in smile_ids:
        top_smiles.append(name2smile[i])
    
    print(top_smiles)
    
    with open(karmadock_score_top_smile_txt, 'w') as outfile:
        for smiles in top_smiles:
            outfile.write(smiles + '\n')
            
    sorted_df = df.sort_values(by='karma_score', ascending=False)
    sorted_df.insert(loc=1, column='smiles', value=[name2smile[i] for i in sorted_df['pdb_id']])
    sorted_df.rename(columns={'pdb_id': 'molecule_id'}, inplace=True)
    return sorted_df
    # sorted_df.to_csv(os.path.join(os.path.dirname(text_file) , 'karmadock_prediction_dealt.csv') , index=False)
    

def karmadock_prediction(protein_file , crystal_ligand_file , ligand_smi, out_dir,cuda_number):
        print('--shm-size="8g"')
        cmd = 'docker run -it --rm -v ./:/home  --shm-size="8g" hierarchicalvs:v5 /bin/bash -c \
                "source activate && conda activate karmadock && cd /app/Models/KarmaDock/utils  && unset LD_LIBRARY_PATH && export LD_LIBRARY_PATH=/opt/conda/envs/karmadock/lib:$LD_LIBRARY_PATH  && \
                CUDA_VISIBLE_DEVICES=%s python -u virtual_screening.py \
                --ligand_smi %s \
                --protein_file %s \
                --crystal_ligand_file %s \
                --out_dir %s \
                --score_threshold 0 \
                --batch_size 64 \
                --random_seed 2023 "' % (cuda_number, ligand_smi, protein_file, crystal_ligand_file, out_dir)
        subprocess.call(cmd, shell=True)


def carsidock_prediction(protein_file , crystal_ligand_file , ligand_sdf, out_dir , cuda_number):
        cmd = 'docker run -it --rm -v ./:/home --shm-size="8g" hierarchicalvs:v5 /bin/bash -c \
                "cd /app/Models/CarsiDock && CUDA_VISIBLE_DEVICES=%s python -u run_screening.py \
                --cuda_convert \
                --pdb_file %s \
                --reflig %s \
                --ligands %s \
                --output_dir %s"' %(cuda_number, protein_file , crystal_ligand_file , ligand_sdf,out_dir)
        subprocess.call(cmd, shell=True)
    

def change_path(old_path, old_substring='/home', new_substring='.'):
    if old_path.startswith(old_substring):
        new_path = old_path.replace(old_substring, new_substring, 1)
        return new_path
    else:
        return "原始路径不包含指定子字符串"
    
def change_path_reverse(old_path, old_substring='.', new_substring='/home'):
    if old_path.startswith(old_substring):
        new_path = old_path.replace(old_substring, new_substring, 1)
        return new_path
    else:
        return "原始路径不包含指定子字符串"
    
          
def hierarchicalvs():

    try:
        
        parser = argparse.ArgumentParser(description='Process input and output protein files.')
        parser.add_argument('-p', '--protein_file', required=True, help='the relative path of the protein file.')
        parser.add_argument('-l', '--ligand_file', required=True, help='the relative path of the ligand file.')
        parser.add_argument('-cl', '--compound_library', required=True, help='the relative path of the compound library.')
        parser.add_argument('-o', '--output_path', required=True, help='the storage path for the output file')
        parser.add_argument('-n', '--topN', required=False, default = 50 , type = int , help='the number of molecules for CarsiDock')
        parser.add_argument('-P', '--Poses_keep', required=False, default = False ,  help='whether to keep the CarsiDock-based poses')
        args = parser.parse_args()

        protein_file = args.protein_file
        ligand_file = args.ligand_file
        compound_library = args.compound_library
        output_path = args.output_path
        topN = args.topN
        Poses_keep = args.Poses_keep
        
        protein_file_docker = change_path_reverse(protein_file)
        crystal_ligand_file_docker = change_path_reverse(ligand_file)
        ligand_smi_docker = change_path_reverse(compound_library)
        out_dir_docker = change_path_reverse(output_path)
     
        
        cuda_number = 0 
         
        karmadock_prediction(protein_file_docker , crystal_ligand_file_docker , ligand_smi_docker, out_dir_docker,cuda_number)
        os.rename(os.path.join(change_path(out_dir_docker) , 'score.csv') , os.path.join(change_path(out_dir_docker) , 'karmadock_prediction.csv'))
        karmadock_prediction_file = os.path.join(change_path(out_dir_docker) , 'karmadock_prediction.csv')
        
        ligand_smi_local = change_path(ligand_smi_docker)
        karmadock_score_top_smile_txt = os.path.join(out_dir_docker , 'karmadock_score_top%s_smile.txt'%topN)
        
        data_dealt = process_karmadock_prediction_files(ligand_smi_local, karmadock_prediction_file,  change_path(karmadock_score_top_smile_txt)  ,topN)
        data_dealt.to_csv(os.path.join(os.path.dirname(change_path(karmadock_score_top_smile_txt)) , 'karmadock_prediction_dealt.csv') , index=False)
        
        
       
        carsidock_prediction(protein_file_docker , crystal_ligand_file_docker , karmadock_score_top_smile_txt, out_dir_docker,cuda_number)
               
        carsidock_prediction_file = change_path(os.path.join(out_dir_docker , 'score.dat'))
        karmadock_score_top_smile_txt_local = change_path(os.path.join(out_dir_docker , 'karmadock_score_top%s_smile.txt'%topN))
        get_final_smi_score(carsidock_prediction_file , karmadock_score_top_smile_txt_local)
              
        protein_file_name = os.path.basename(protein_file_docker)
        get_final_ligand_poses(carsidock_prediction_file , protein_file_name , change_path(protein_file_docker))
        
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'carsidock_prediction.csv')):
            os.rename(os.path.join(change_path(out_dir_docker), 'carsidock_prediction.csv')   , os.path.join(change_path(out_dir_docker), 'rtmscore_prediction.csv'))
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'inchiKey2smi.csv')):
            os.remove(os.path.join(change_path(out_dir_docker), 'inchiKey2smi.csv'))
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'karmadock_prediction.csv')):
            os.remove(os.path.join(change_path(out_dir_docker), 'karmadock_prediction.csv'))
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'karmadock_prediction_dealt.csv')):
            os.rename(os.path.join(change_path(out_dir_docker), 'karmadock_prediction_dealt.csv') , os.path.join(change_path(out_dir_docker), 'karmadock_prediction.csv'))
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'score.dat')):
            os.remove(os.path.join(change_path(out_dir_docker), 'score.dat'))
            
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'karmadock_score_top%s_smile.txt'%topN)):
            os.remove(os.path.join(change_path(out_dir_docker), 'karmadock_score_top%s_smile.txt'%topN))
            
        if os.path.exists(os.path.join(change_path(out_dir_docker), 'top10_poses_interaction')):
            os.system('rm -rf %s'%os.path.join(change_path(out_dir_docker), 'top10_poses_interaction'))
        if not Poses_keep:
            os.remove(os.path.join(change_path(out_dir_docker), 'ligand_poses.zip'))
         
            
    except Exception as e:
    
        print("error:", e)
        sys.exit(1)








# ligand_smi = './example/active_decoys.smi'
# protein_file = './example/6PYR_optimal_pocket.pdb'
# crystal_ligand_file = './example/crystal_ligand.mol2'
# out_dir = './out_dir'
# topN=10
# Poses_keep = True
# hierarchical_prediction(change_path_reverse(protein_file) , change_path_reverse(crystal_ligand_file), change_path_reverse(ligand_smi), change_path_reverse(out_dir), topN=topN, Poses_keep = Poses_keep)







import sys
import csv
import oddt
from oddt import fingerprints
import numpy as np
from oddt import interactions
import os,traceback
# from pymol import cmd


# def compose_protein_and_ligand(path_protein, path_ligand, path_output):
#     cmd.reinitialize()
#     cmd.load(path_protein, 'protein')
#     cmd.load(path_ligand, 'ligand')
#     cmd.create("complex", "protein or ligand" )
#     cmd.save(os.path.join(path_output , 'complex.pdb'), "complex")

def cal_ifp(path_protein, path_ligand, path_output , i):
    
    protein_name  = os.path.basename(path_protein)
    ligand_name =  os.path.basename(path_ligand)
    os.system('cp %s %s'%(path_ligand , path_output))
    os.rename(os.path.join(path_output , ligand_name) ,   os.path.join(path_output , 'top%s.sdf'%i) )
    
    if os.path.exists(path_ligand):
        try:
            protein = next(oddt.toolkit.readfile('pdb' , path_protein))
            protein.protein = True
            ligand = next(oddt.toolkit.readfile('sdf' , path_ligand))
            new0 = open(os.path.join(path_output , 'interaction.csv'), 'w', newline='')
            mycsv0 = csv.writer(new0)
            mycsv0.writerow(
                ['protein_name', 'ligand_name', 'type','res_name' ,'res_chain', 'res_idx_PDB', 'res_idx', 'atom_type1', 
                 'isacceptor', 'x', 'y', 'z', 'atom_type3', 'x2', 'y2', 'z2','strict/parallel','perpendicular']
            )

            # 氢键
            a, b ,strict = interactions.hbonds(protein,ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'hbond.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    # protein
                    atom = protein.atoms[a[i][0]]
                    residue = protein.residues[int(a[i][9])]
                    res_name = a[i][11]
                    res_idx = a[i][9]
                    res_idx_PDB = a[i][10]

                    res_chain = residue.chain
                    atom_type1 = a[i][5]

                    isacceptor = a[i][13]
                    x = a[i][1][0]
                    y = a[i][1][1]
                    z =a[i][1][2]

                    # ligand
                    atom_type3 = b[i][5]
                    x2 = b[i][1][0]
                    y2 = b[i][1][1]
                    z2 = b[i][1][2]

                    st = strict[i]

                    # 排除那部分与反应氨基酸形成的相互作用

                    mycsv.writerow(
                        [protein_name,ligand_name,'hbond',res_name,res_chain,res_idx_PDB,res_idx,atom_type1,isacceptor,x,y,z,atom_type3,x2,y2,z2,st]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'hbond', res_name, res_chain, res_idx_PDB, res_idx, atom_type1, isacceptor, x, y, z, atom_type3, x2, y2, z2, st]
                    )

            # hbond = a.__len__()

            # 卤素键
            a, b, strict = interactions.halogenbonds(protein, ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'halo.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    # protein
                    atom = protein.atoms[a[i][0]]
                    residue = protein.residues[int(a[i][9])]
                    res_name = a[i][11]
                    res_idx = a[i][9]
                    res_idx_PDB = a[i][10]

                    res_chain = residue.chain
                    atom_type1 = a[i][5]

                    isacceptor = a[i][13]
                    x = a[i][1][0]
                    y = a[i][1][1]
                    z = a[i][1][2]

                    # ligand
                    atom_type3 = b[i][5]
                    x2 = b[i][1][0]
                    y2 = b[i][1][1]
                    z2 = b[i][1][2]

                    st = strict[i]

                    mycsv.writerow(
                        [protein_name, ligand_name, 'halo',res_name, res_chain, res_idx_PDB, res_idx, atom_type1,
                         isacceptor, x, y, z, atom_type3, x2, y2, z2, st]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'halo', res_name, res_chain, res_idx_PDB, res_idx, atom_type1, 
                         isacceptor, x, y, z, atom_type3, x2, y2, z2, st]
                    )
            # halo = a.__len__()

            # pi-pi
            a,b ,c,d = interactions.pi_stacking(protein,ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'pipi.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    # protein
                    x = a[i][0][0]
                    y = a[i][0][1]
                    z = a[i][0][2]
                    residue = protein.residues[int(a[i][2])]
                    res_idx = a[i][2]
                    res_idx_PDB = a[i][3]
                    res_name = a[i][4]
                    res_chain = residue.chain
                    atom_type1 = '!'


                    isacceptor = '!'

                    # ligand
                    atom_type3 = '!'
                    x2 = b[i][0][0]
                    y2 = b[i][0][1]
                    z2 = b[i][0][2]

                    parallel = c[i]
                    perpendicular = d[i]

                    mycsv.writerow(
                        [protein_name, ligand_name, 'pipi',res_name, res_chain, res_idx_PDB, res_idx,atom_type1, isacceptor,x, y, z, atom_type3,x2, y2, z2, parallel,perpendicular]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'pipi', res_name, res_chain, res_idx_PDB, res_idx, atom_type1, isacceptor, x, y, z, atom_type3, x2, y2, z2, parallel, perpendicular]
                    )

            # pi_pi = a.__len__()

            # pi_cation
            a, b, c = interactions.pi_cation(protein, ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'pi-cation.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    if a[i][4] == '':
                        # 环在配体上
                        # ring
                        x2 = a[i][0][0]
                        y2 = a[i][0][1]
                        z2 = a[i][0][2]
                        atom_type3 = '!'
                        isacceptor = '!'
                        # cation
                        x = b[i][1][0]
                        y = b[i][1][1]
                        z = b[i][1][2]

                        atom = protein.atoms[b[i][0]]
                        residue = protein.residues[int(b[i][9])]
                        res_name = b[i][11]
                        res_idx = b[i][9]
                        res_idx_PDB = b[i][10]

                        res_chain = residue.chain
                        atom_type1 = b[i][5]


                    else:
                        # 环在蛋白上
                        x = a[i][0][0]
                        y = a[i][0][1]
                        z = a[i][0][2]
                        residue = protein.residues[int(a[i][2])]
                        res_idx = a[i][2]
                        res_idx_PDB = a[i][3]
                        res_name = a[i][4]
                        res_chain = residue.chain
                        atom_type1 = '!'


                        isacceptor = '!'

                        # cation
                        atom_type3 = b[i][5]
                        x2 = b[i][1][0]
                        y2 = b[i][1][1]
                        z2 = b[i][1][2]


                    st = c[i]

                    mycsv.writerow(
                        [protein_name, ligand_name, 'pi_cation',res_name, res_chain, res_idx_PDB, res_idx, atom_type1, 
                         isacceptor, x, y, z, atom_type3, x2, y2, z2, st]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'pi_cation', res_name, res_chain, res_idx_PDB, res_idx, atom_type1,
                         isacceptor, x, y, z, atom_type3, x2, y2, z2, st]
                    )

            # pi_cation = a.__len__()

            # 盐桥
            a, b, = interactions.salt_bridges(protein, ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'salt_bridge.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    # protein
                    atom = protein.atoms[a[i][0]]
                    residue = protein.residues[int(a[i][9])]
                    res_name = a[i][11]
                    res_idx = a[i][9]
                    res_idx_PDB = a[i][10]

                    res_chain = residue.chain
                    atom_type1 = a[i][5]

                    isacceptor = a[i][13]
                    x = a[i][1][0]
                    y = a[i][1][1]
                    z = a[i][1][2]

                    # ligand
                    atom_type3 = b[i][5]
                    x2 = b[i][1][0]
                    y2 = b[i][1][1]
                    z2 = b[i][1][2]

                    mycsv.writerow(
                        [protein_name, ligand_name,'salt_bridge', res_name, res_chain, res_idx_PDB, res_idx, atom_type1, 
                         isacceptor, x, y, z, atom_type3, x2, y2, z2]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'salt_bridge', res_name, res_chain, res_idx_PDB, res_idx, atom_type1, 
                         isacceptor, x, y, z, atom_type3, x2, y2, z2]
                    )
            # bridge = a.__len__()

            # 疏水
            a, b = interactions.hydrophobic_contacts(protein, ligand)
            if a.__len__() != 0:
                new = open(os.path.join(path_output , 'hydrophobic.csv'), 'w', newline='')
                mycsv = csv.writer(new)
                for i in range(len(a)):
                    # protein
                    atom = protein.atoms[a[i][0]]
                    residue = protein.residues[int(a[i][9])]
                    res_name = a[i][11]
                    res_idx = a[i][9]
                    res_idx_PDB = a[i][10]

                    res_chain = residue.chain
                    atom_type1 = a[i][5]
                    isacceptor = a[i][13]
                    x = a[i][1][0]
                    y = a[i][1][1]
                    z = a[i][1][2]

                    # ligand
                    atom_type3 = b[i][5]
                    x2 = b[i][1][0]
                    y2 = b[i][1][1]
                    z2 = b[i][1][2]

                    mycsv.writerow(
                        [protein_name, ligand_name, 'hydrophobic',res_name, res_chain, res_idx_PDB, res_idx, atom_type1, 
                         isacceptor, x, y, z, atom_type3, x2, y2, z2]
                    )
                    mycsv0.writerow(
                        [protein_name, ligand_name, 'hydrophobic', res_name, res_chain, res_idx_PDB, res_idx, atom_type1,
                         isacceptor, x, y, z, atom_type3, x2, y2, z2]
                    )
            new0.close()
        except:
            traceback.print_exc()

         # hydro = a.__len__()
         



import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="hierarchicalvs",  # 模块名称
    version="1.1",  # 当前版本
    author="shukaigu",  # 作者
    author_email="3160101168@zju.edu.cn",  # 作者邮箱
    description="a fully AI-driven virtual screening package",  # 模块简介
    long_description=long_description,  # 模块详细介绍
    long_description_content_type="text/markdown",  # 模块详细介绍格式
    packages=setuptools.find_packages(),  # 自动找到项目中导入的模块
    # 模块相关的元数据
    package_data={
        'hierarchicalvs': ['**/*'],  # 指定要包含的文件类型
        },
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    # 依赖模块
    entry_points={
        'console_scripts': [
            'hierarchicalvs=hierarchicalvs.hierarchicalvs:hierarchicalvs',
        ],
    },
    install_requires=[
        'pillow',
        'rdkit==2022.09.1',
        'oddt',
    ],
    python_requires='>=3.6',
)